@@ -181,11 +181,13 @@ class MainGroup(Group):
             network_timeout=kwargs["network_timeout"],
             config_path=Path(kwargs["neuromation_config"]),
             trace=kwargs["trace"],
+            force_trace_all=kwargs["x_trace_all"],
             trace_hide_token=hide_token_bool,
             command_path="",
             command_params=[],
             skip_gmp_stats=kwargs["skip_stats"],
             show_traceback=show_traceback,
+            iso_datetime_format=kwargs["iso_datetime_format"],
         )
         handler.setConsole(root.err_console)
         ctx.obj = root
@@ -383,6 +385,12 @@ def print_options(
     is_flag=True,
     help="Trace sent HTTP requests and received replies to stderr.",
 )
+@option(
+    "--x-trace-all",
+    hidden=True,
+    is_flag=True,
+    help="Force distribute tracing in all HTTP requests.",
+)
 @option(
     "--hide-token/--no-hide-token",
     is_flag=True,
@@ -404,6 +412,12 @@ def print_options(
         "environment variables, etc."
     ),
 )
+@option(
+    "--iso-datetime-format/--no-iso-datetime-format",
+    is_flag=True,
+    default=False,
+    help=("Use ISO 8601 format for printing date and time"),
+)
 @click.pass_context
 def cli(
     ctx: click.Context,
@@ -415,8 +429,10 @@ def cli(
     disable_pypi_version_check: bool,
     network_timeout: float,
     trace: bool,
+    x_trace_all: bool,
     hide_token: Optional[bool],
     skip_stats: bool,
+    iso_datetime_format: bool,
 ) -> None:
     #   ▇ ◣
     #   ▇ ◥ ◣
