@@ -1,6 +1,7 @@
 import asyncio
 import os
 import re
+import shlex
 import subprocess
 import sys
 import uuid
@@ -124,6 +125,38 @@ def test_job_run(helper: Helper) -> None:
     helper.kill_job(job_id, wait=False)
 
 
+@pytest.mark.e2e
+def test_job_rerun(helper: Helper) -> None:
+    captured = helper.run_cli(
+        [
+            "-q",
+            "job",
+            "run",
+            UBUNTU_IMAGE_NAME,
+            'bash -c "exit 0"',
+        ]
+    )
+    job_id = captured.out
+
+    helper.wait_job_change_state_to(
+        job_id, JobStatus.SUCCEEDED, stop_state=JobStatus.FAILED
+    )
+    captured = helper.run_cli(
+        [
+            "-q",
+            "job",
+            "generate-run-command",
+            job_id,
+        ]
+    )
+    args = shlex.split(captured.out)
+    captured = helper.run_cli(["-q", *args[1:]])
+    job_id = captured.out
+    helper.wait_job_change_state_to(
+        job_id, JobStatus.SUCCEEDED, stop_state=JobStatus.FAILED
+    )
+
+
 @pytest.mark.e2e
 def test_job_description(helper: Helper) -> None:
     # Remember original running jobs
@@ -899,7 +932,7 @@ def test_e2e_job_top(helper: Helper) -> None:
             returncode = 0
 
         print("STDOUT", stdout)
-        if "TIMESTAMP" in stdout and "MEMORY (MB)" in stdout:
+        if "MEMORY (MB)" in stdout:
             # got response from job top telemetery
             returncode = 0
             break
@@ -1154,3 +1187,34 @@ def test_job_disk_volume(
         job_id = match.group(1)
 
         helper.wait_job_change_state_to(job_id, JobStatus.SUCCEEDED, JobStatus.FAILED)
+
+
+@pytest.mark.e2e
+def test_job_disk_volume_named(
+    helper: Helper, disk_factory: Callable[[str, str], ContextManager[str]]
+) -> None:
+    disk_name = f"test-disk-{os.urandom(5).hex()}"
+
+    with disk_factory("1G", disk_name):
+        bash_script = 'echo "test data" > /mnt/disk/file && cat /mnt/disk/file'
+        command = f"bash -c '{bash_script}'"
+        captured = helper.run_cli(
+            [
+                "job",
+                "run",
+                "--life-span",
+                "1m",  # Avoid completed job to block disk from cleanup
+                "-v",
+                f"disk:{disk_name}:/mnt/disk:rw",
+                "--no-wait-start",
+                UBUNTU_IMAGE_NAME,
+                command,
+            ]
+        )
+
+        out = captured.out
+        match = re.match("Job ID: (.+)", out)
+        assert match is not None, captured
+        job_id = match.group(1)
+
+        helper.wait_job_change_state_to(job_id, JobStatus.SUCCEEDED, JobStatus.FAILED)
